#pragma once
#include "GLFW/glfw3.h"
#include <vector>
#include "../PerfCounter.h"

#include "Renderable.h"
#include "Memory/Framebuffer.h"
#include "../Scene/Components.h"
#include"../Scene/Scene.h"
#include "Primitives/Camera.h"
#include "OpenglAPI.h"

namespace YoggieEngine {

	struct RendererConfig {
		int ScreenWidth, ScreenHeight;
		glm::vec3 ClearColor;
		bool DepthTest; 
	};

	class Renderer {
	public:
		Renderer();
		~Renderer();


		// Forward pass
		/*
		forwardShader.Use();
		forwardShader.setUniformVec3("Color", renderComponent.color);
		forwardShader.setUniformMat4("M", transform.LocalTransform);
		forwardShader.setUniformMat4("V", MainCamera.view);
		forwardShader.setUniformMat4("P", MainCamera.projection);

		OpenGLApi::DrawTriangles(renderComponent);
		*/

		void Render(Scene& scene, Camera MainCamera) {
			
			MainCamera.projection = glm::perspective(glm::radians(65.0f), ((float)width/(float)height), 0.0001f, 1000.0f);
			int oldviewport[4];
			glGetIntegerv(GL_VIEWPORT, oldviewport);
			glViewport(0, 0, width, height);


			glBindFramebuffer(GL_FRAMEBUFFER, gBuffer);
			glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			// Render skybox if the scene has one 

			SkyboxShader.Use();
			SkyboxShader.setUniformMat4("projection", MainCamera.projection);
			SkyboxShader.setUniformMat4("view", glm::mat4(glm::mat3(MainCamera.view)));

			if (!skyboxVAO) {
				unsigned int VBO;
				glGenVertexArrays(1, &skyboxVAO);
				glGenBuffers(1, &VBO);
				glBindVertexArray(skyboxVAO);
				glBindBuffer(GL_ARRAY_BUFFER, VBO);
				glBufferData(GL_ARRAY_BUFFER, sizeof(skyboxVertices), &skyboxVertices, GL_STATIC_DRAW);
				glEnableVertexAttribArray(0);
				glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (void*)0);
					
			}

			OpenGLApi::DrawCubeMap( skyboxVAO, skybox);
			


		
			auto renderables = scene.getReg().view<TransformComponent, Render3DComponent>();
			gBufferShader.Use();
			for (auto renderable : renderables) {
				auto entity = Entity(renderable, &scene);
				auto& renderComponent = entity.GetComponent<Render3DComponent>();
				auto& transform = entity.GetComponent<TransformComponent>();
				
				// Geometry pass 
				gBufferShader.setUniformVec3("Color", renderComponent.color);
				gBufferShader.setUniformMat4("Model", transform.GetTransform());
				gBufferShader.setUniformMat4("View", MainCamera.view);
				gBufferShader.setUniformMat4("Projection", MainCamera.projection);
				OpenGLApi::DrawTriangles(renderComponent);

				
			}

			// Light pass
			glBindFramebuffer(GL_FRAMEBUFFER, m_framebuffer.GetId());
			lightingPassShader.Use();
			lightingPassShader.setUniformInt("gPosition", 0);
			lightingPassShader.setUniformInt("gNormal", 1);
			lightingPassShader.setUniformInt("gColorSpec", 2);

			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, gPosition);

			glActiveTexture(GL_TEXTURE1);
			glBindTexture(GL_TEXTURE_2D, gNormal);

			glActiveTexture(GL_TEXTURE2);
			glBindTexture(GL_TEXTURE_2D, gColorSpec);

			auto lights = scene.getReg().view<LightComponent, TransformComponent>();
			unsigned int lightnr = 0;
			for (auto light : lights) {
				auto lightComponent = Entity(light, &scene).GetComponent<LightComponent>();
				auto transformComponent = Entity(light, &scene).GetComponent<TransformComponent>();

				auto name = "lights[" + std::to_string(lightnr) + "]";
				lightingPassShader.setUniformVec3(name + ".Position",transformComponent.Position );
				lightingPassShader.setUniformVec3(name + ".Color", lightComponent.Color);

				const float linear = 0.7f;
				const float quadratic = 1.8f;
				lightingPassShader.setUniformFloat(name + ".Linear", linear);
				lightingPassShader.setUniformFloat(name + ".Quadratic", quadratic);

				lightnr++;

			}

			if (quadVAO == 0)
			{
				float quadVertices[] = {
					-1.0f,	 1.0f, 0.0f, 0.0f, 1.0f,
					-1.0f,	-1.0f, 0.0f, 0.0f, 0.0f,
					 1.0f,	 1.0f, 0.0f, 1.0f, 1.0f,
					 1.0f,	-1.0f, 0.0f, 1.0f, 0.0f,
				};
				unsigned int quadVBO;
				// setup plane VAO ;
				glGenVertexArrays(1, &quadVAO);
				glGenBuffers(1, &quadVBO);
				glBindVertexArray(quadVAO);
				glBindBuffer(GL_ARRAY_BUFFER, quadVBO);
				glBufferData(GL_ARRAY_BUFFER, sizeof(quadVertices), &quadVertices, GL_STATIC_DRAW);
				glEnableVertexAttribArray(0);
				glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 5 * sizeof(float), (void*)0);
				glEnableVertexAttribArray(1);
				glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 5 * sizeof(float), (void*)(3 * sizeof(float)));
			}

			glBindVertexArray(quadVAO);
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
			glBindVertexArray(0);


			// Copy GBuffer
			glBindFramebuffer(GL_READ_FRAMEBUFFER, gBuffer);
			glBindFramebuffer(GL_DRAW_BUFFER, m_framebuffer.GetId());
			glBlitFramebuffer(0, 0, width, height, 0, 0, width, height, GL_DEPTH_BUFFER_BIT, GL_NEAREST);


			if (transparentVAO == 0) {
				unsigned int transparentVBO;
				float transparentVertices[] = {
					// positions         // texture Coords (swapped y coordinates because texture is flipped upside down)
					0.0f,  0.5f,  0.0f,  0.0f,  1.0f,
					0.0f, -0.5f,  0.0f,  0.0f,  0.0f,
					1.0f, -0.5f,  0.0f,  1.0f,  0.0f,

					0.0f,  0.5f,  0.0f,  0.0f,  1.0f,
					1.0f, -0.5f,  0.0f,  1.0f,  0.0f,
					1.0f,  0.5f,  0.0f,  1.0f,  1.0f
				};

				glGenVertexArrays(1, &transparentVAO);
				glGenBuffers(1, &transparentVBO);
				glBindVertexArray(transparentVAO);
				glBindBuffer(GL_ARRAY_BUFFER, transparentVBO);
				glBufferData(GL_ARRAY_BUFFER, sizeof(transparentVertices), transparentVertices, GL_STATIC_DRAW);
				glEnableVertexAttribArray(0);
				glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 5 * sizeof(float), (void*)0);
				glEnableVertexAttribArray(1);
				glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 5 * sizeof(float), (void*)(3 * sizeof(float)));
				glBindVertexArray(0);

			}

			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

			BlendingShader.Use();
			glBindVertexArray(transparentVAO);
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, grassTexture.GetID());

			BlendingShader.setUniformMat4("V", MainCamera.view);
			BlendingShader.setUniformMat4("P", MainCamera.projection);
			for (unsigned int i = 0; i < vegetation.size(); i++) {
				
				auto translation = glm::translate(glm::mat4(1.0f), vegetation[i]) ;
				BlendingShader.setUniformMat4("M", translation);
				glDrawArrays(GL_TRIANGLES, 0, 6);

			}

			glBindVertexArray(0);

			glDisable(GL_BLEND);

		



			glBindFramebuffer(GL_FRAMEBUFFER, 0);
			glViewport(oldviewport[0], oldviewport[1], oldviewport[2], oldviewport[3]);
			

		}
			
		void setCurrentFrameBuffer(const Framebuffer& fb);
		void setClearColor(const glm::vec3& ClearColor);


		Framebuffer& getCurrentFrameBuffer() { return m_framebuffer; }

	private:
		Framebuffer m_framebuffer;
		int width, height;
		glm::vec3 m_clearColor;
		bool m_depthTest;

		void CreateGBuffer();

		Shader forwardShader;

		// deferred rending parameters
		unsigned int gBuffer, gPosition, gNormal, gColorSpec, gDepth;
		Shader lightingPassShader;
		Shader gBufferShader;
		Shader SkyboxShader;
		Shader postProcessingShader;

		// blending
		Shader BlendingShader;
		CubeMap skybox;
		Texture grassTexture = Texture("build/Debug/Texture/grass.png");

		unsigned int transparentVAO = 0;
		unsigned int skyboxVAO = 0;
		unsigned int quadVAO = 0;
		float skyboxVertices[36*3]{
			// positions          
			-1.0f,  1.0f, -1.0f,
			-1.0f, -1.0f, -1.0f,
			 1.0f, -1.0f, -1.0f,
			 1.0f, -1.0f, -1.0f,
			 1.0f,  1.0f, -1.0f,
			-1.0f,  1.0f, -1.0f,
			-1.0f, -1.0f,  1.0f,
			-1.0f, -1.0f, -1.0f,
			-1.0f,  1.0f, -1.0f,
			-1.0f,  1.0f, -1.0f,
			-1.0f,  1.0f,  1.0f,
			-1.0f, -1.0f,  1.0f,
			 1.0f, -1.0f, -1.0f,
			 1.0f, -1.0f,  1.0f,
			 1.0f,  1.0f,  1.0f,
			 1.0f,  1.0f,  1.0f,
			 1.0f,  1.0f, -1.0f,
			 1.0f, -1.0f, -1.0f,
			-1.0f, -1.0f,  1.0f,
			-1.0f,  1.0f,  1.0f,
			 1.0f,  1.0f,  1.0f,
			 1.0f,  1.0f,  1.0f,
			 1.0f, -1.0f,  1.0f,
			-1.0f, -1.0f,  1.0f,
			-1.0f,  1.0f, -1.0f,
			 1.0f,  1.0f, -1.0f,
			 1.0f,  1.0f,  1.0f,
			 1.0f,  1.0f,  1.0f,
			-1.0f,  1.0f,  1.0f,
			-1.0f,  1.0f, -1.0f,
			-1.0f, -1.0f, -1.0f,
			-1.0f, -1.0f,  1.0f,
			 1.0f, -1.0f, -1.0f,
			 1.0f, -1.0f, -1.0f,
			-1.0f, -1.0f,  1.0f,
			 1.0f, -1.0f,  1.0f
		};
		std::vector<glm::vec3> vegetation = {
	glm::vec3(-1.5f, 0.0f, -0.48f),
	glm::vec3(1.5f, 0.0f, 0.51f),
	glm::vec3(0.0f, 0.0f, 0.7f),
	glm::vec3(-0.3f, 0.0f, -2.3f)
		};

	};
}
