#include <YoggieEngine.h>
#include "InputManager.h"

namespace YoggieEngine  {
	double cursor_pos_x;
	double cursor_pos_y;
	int key_status[YOGGIE_KEY_LAST];
	int mouseButton_status[7];


	void key_callback(GLFWwindow* window, int key, int scancode, int action, int mods);
	void cursorPos_callback(GLFWwindow* window, double xpos, double ypos);
	void cursorEnter_callback(GLFWwindow* window, int entered);
	void mouseButton_callback(GLFWwindow* window, int button, int action, int mods);


	void init_inputSystem(NativeWindow* window) {
		cursor_pos_x = 0;
		cursor_pos_y = 0;
		glfwSetKeyCallback((GLFWwindow*) window->GetHandle(), key_callback);
		glfwSetMouseButtonCallback((GLFWwindow*)window->GetHandle(), mouseButton_callback);
		glfwSetCursorPosCallback((GLFWwindow*)window->GetHandle(), cursorPos_callback);
		glfwSetCursorEnterCallback((GLFWwindow*)window->GetHandle(), cursorEnter_callback);
	}

	void ReceiveInput() {
		glfwPollEvents();

	}


	bool keyIsPressed(int key)
	{
		return (key_status[key] == GLFW_PRESS) || (key_status[key] == GLFW_REPEAT);
	}

	bool MouseButtonPressed(int key) 
	{
		return (mouseButton_status[key] == GLFW_PRESS  ) ||  (mouseButton_status[key] == GLFW_REPEAT);

	}


	double getCursorPosX(NativeWindow* window)
	{
		glfwGetCursorPos((GLFWwindow*)window->GetHandle(), &cursor_pos_x, nullptr);
		return cursor_pos_x;
	}

	double getCursorPosY(NativeWindow* window)
	{
		glfwGetCursorPos((GLFWwindow*)window->GetHandle(), nullptr, &cursor_pos_y);
		return cursor_pos_y;
	}

	void key_callback(GLFWwindow* window, int key, int scancode, int action, int mods)
	{
		key_status[key] = action;
	}

	void mouseButton_callback(GLFWwindow* window, int button, int action, int mods)
	{
	
		auto& io = ImGui::GetIO();
		io.AddMouseButtonEvent(button , action == GLFW_PRESS);
		
		
		mouseButton_status[button] = action;
	}



	void cursorPos_callback(GLFWwindow* window, double xpos, double ypos)
	{
		//std::cout << "Cursor moved!" << std::endl;
	}

	void cursorEnter_callback(GLFWwindow* window, int entered)
	{
		if (entered) {
			// window is in focus
			//std::cout << "Cursor Entered!" << std::endl;
		}
		else {
			// window is not in focus
			//std::cout << "Cursor Left!" << std::endl;
		}
	}






}



