#pragma once
#include "Scene.h"

namespace YoggieEngine {
	
	class Entity {
	public:
		Entity() = default;
		Entity(entt::entity e, Scene* scene) ;
		Entity(const Entity& other) = default;

		template<class T, class... argumentType >
		T& AddComponent(argumentType... args) {
			return m_scene->m_registry.emplace<T>(m_entity, args...);
		}

		template<class T>
		T& GetComponent() {
			return m_scene->m_registry.get<T>(m_entity);
		}


		template<class T>
		bool HasComponent() {
			return m_scene->getReg().all_of<T>(m_entity);
		}

		// NOTE: Not Scene context aware!!
		inline bool operator== (Entity& other) {
			return m_entity == other.m_entity;// && other.m_scene == m_scene;
		}

		inline bool isValid() {
			if (m_scene == nullptr)
				return false;

			return m_scene->m_registry.valid(m_entity);
		}

	private:
		entt::entity m_entity;
		Scene* m_scene;

	};

}
